@@ -7,7 +7,7 @@ use Carp;
 use DBI;
 use SQL::Abstract;
 
-our $VERSION = '0.14';
+our $VERSION = '0.16';
 
 =head1 NAME
 
@@ -31,7 +31,7 @@ Each table then becomes a Perl package.
   MySchema->Table(Activity   => Activity   => qw/act_id/);
 
 Declare associations in UML style
-( C<< [table1 role1 arity1 join1], [table2...] >>).
+( C<< [table1 role1 multiplicity1 join1], [table2...] >>).
 
   MySchema->Association([qw/Activity   activities * emp_id/],
                         [qw/Employee   employee   1 emp_id/]);
@@ -99,14 +99,20 @@ Get a list of employee names in age order
 
   my $ageLst = Employee->select([qw/lastname firstname/], {}, ['d_birth']);
 
-Print some info from employees, either by direct access to the
-hashref, or by a method call (handled via AUTOLOAD). Because of the
+Print some info from employees. Because of the
 'fromDB' handler associated with column type 'date', column 'd_birth'
 has been automatically converted to display format.
 
   foreach my $emp (@$empl_D) {
-    printf "%s (%s) : %s\n", 
-      $emp->{firstname}, $emp->lastname, $emp->d_birth;
+    print "$emp->{firstname} $emp->{lastname}, born $emp->{d_birth}\n";
+  }
+
+Same thing, but using method calls instead of direct access to the
+hashref (must enable AUTOLOAD in the table or the whole schema)
+
+  Employee->Autoload(1); # or MySchema->Autoload(1)
+  foreach my $emp (@$empl_D) {
+    printf "%s %s, born %s\n", $emp->firstname, $emp->lastname, $emp->d_birth;
   }
 
 Follow the joins through role methods
@@ -133,7 +139,7 @@ we need to expand the objects :
 
 Select associated tables directly from a database join, 
 in one single SQL statement (instead of iterating through role methods).
-  
+
   my $lst = MySchema->ViewFromRoles(qw/Employee activities department/)
                     ->select([qw/lastname dept_name d_begin/], 
                              {d_begin => {'>=' => '2000-01-01'}});
@@ -147,7 +153,7 @@ and objects around database tables and records. There are many other
 CPAN modules in this area; perhaps the mere fact that they are so
 numerous demonstrates that there is more than one way to do it, and
 none is obviously the best, so why not propose another one ?
-The L<SEE ALSO> section at the end of this documentation gives 
+The L</"SEE ALSO"> section at the end of this documentation gives 
 some pointers.
 
 C<DBIx::DataModel> is written compactly as one single module,
@@ -165,8 +171,7 @@ simple way to ask for all available methods, and even if you get
 there, it is not possible to distinguish which of those methods
 encapsulate relevant data). Therefore C<DBIx::DataModel> does not
 insist on OO information hiding; on the contrary, direct access
-to the object hash is encouraged for inspecting the data. In the
-same spirit, transaction handling is left to the client code.
+to the object hash is encouraged for inspecting the data.
 
 C<DBIx::DataModel> defines an API for accessing the database from
 Perl, but will not create the database itself. So use your best
@@ -200,15 +205,15 @@ to follow the synopsis :
 
 =item 1
 
-declare your tables through the L<Table> method
+declare your tables through the L</"Table"> method
 
 =item 2
 
-declare your UML binary associations through the L<Association> method
+declare your UML binary associations through the L</"Association"> method
 
 =item 3
 
-call the L<select> method on one of your main tables
+call the L</"select"> method on one of your main tables
 
 =item 4 
 
@@ -270,7 +275,7 @@ Creates a new Perl class of name C<$schemaName> that represents a
 database schema.
 That class inherits from C<DBIx::DataModel>.
 The connection to a DBI database handle can be set via the optional C<$dbh>
-argument (but it can also be set or reset later) via the L<dbh> method.
+argument (but it can also be set or reset later) via the L</"dbh"> method.
 
 =cut
 
@@ -361,8 +366,8 @@ database Table. That class inherits from C<MySchema>.
 C<< $dbTable >> should contain the name of the table in the database.
 C<< @primKey >> should contain the name(s) of the column(s) holding the primary
 key for that table. This info will be used for interpreting arguments
-to the L<fetch> method, and for filling WHERE clauses in the SQL 
-generated by the L<update> method.
+to the L</"fetch"> method, and for filling WHERE clauses in the SQL 
+generated by the L</"update"> method.
 
 
 =cut
@@ -387,10 +392,10 @@ sub Table {
 
 =head4 View
 
-  MySchema->View($viewName =>$columns => $dbTables => \%where, @parentTables)
+  MySchema->View($viewName, $columns, $dbTables, \%where, @parentTables)
 
 Creates a new Perl class of name C<$viewName> that represents a
-SQL SELECT request. The only method of that class is the L<select> 
+SQL SELECT request. The only method of that class is the L</"select"> 
 method, which will 
 
 =over
@@ -406,9 +411,9 @@ apply the 'fromDB' handlers of the parent tables to those records;
 
 =item *
 
-bless the results into objects of C<MyView> that inherit the
+bless the results into objects of C<$viewName> that inherit the
 role methods of each of the @parentTables (see method
-L<Association> below).
+L</"Association"> below).
 
 =back
 
@@ -420,7 +425,7 @@ Views are useful to build queries with specific SQL clauses like for example
      {c1 => 'foo', c2 => {-like => 'bar%'}},
      @parentTables)
 
-See L<SQL::Abstract> and the L<select> method below for a complete
+See L<SQL::Abstract> and the L</"select"> method below for a complete
 description of what to put in the C<%where> argument. For the moment,
 just consider the following example: 
 
@@ -474,30 +479,30 @@ sub View {
 
 =head4 Association
 
-  MySchema->Association([$table1, $role1, $arity1, @columns1], 
-                        [$table2, $role2, $arity2, @columns2]);
+  MySchema->Association([$table1, $role1, $multiplicity1, @columns1], 
+                        [$table2, $role2, $multiplicity2, @columns2]);
 
 Declares an association between two tables (or even two instances of
 the same table), in a UML-like fashion. Each side of the association
 specifies its table, the "rolename" of of this table in the
-association, the arity (also called "multiplicity"), and the name of
+association, the multiplicity, and the name of
 the column or list of columns that technically implement the
-association as a database join. Arities should be written in the UML form
+association as a database join. Multiplicities should be written in the UML form
 '0..*', '1..*', '0..1', etc. (minimum .. maximum number of occurrences); this
 will influence how role methods and views are implemented,
-as explained below. Arity '*' is a shortcut for '0..*', and arity
+as explained below. Multiplicity '*' is a shortcut for '0..*', and multiplicity
 '1' is a shortcut for '1..1'. Role names should be chosen so as 
 to avoid conflicts with column names in the same table.
 
 As a result of the association declaration, the Perl class
 corresponding to C<< $table1 >> will get an additional method named
 C<< $role2 >> for accessing the associated object(s) in C<< $table2 >>; 
-that method normally returns an arrayref, unless C<< $arity2 >>
+that method normally returns an arrayref, unless C<< $multiplicity2 >>
 has maximum '1' (in that case the return value is a single object
 ref).  Of course, C<< $table2 >> conversely gets a method named 
 C<< $role1 >>. 
 
-To understand why tables and roles are crossed, look a the UML picture :
+To understand why tables and roles are crossed, look at the UML picture :
 
   +--------+                     +--------+
   |        | *              0..1 |        |
@@ -524,7 +529,7 @@ which will implicitly perform a
   SELECT * FROM Activity WHERE emp_id = $anEmployee->{emp_id}
 
 The role method can also accept additional parameters
-in L<SQL::Abstract> format (see also the L<select> method
+in L<SQL::Abstract> format (see also the L</"select"> method
 in this module). So for example
 
   my @acts = $anEmployee->activities(-columns => [qw/act_name salary/], 
@@ -537,7 +542,7 @@ would perform the following SQL request :
     isActive = 'Y'
 
 If the role method is called without any parameters, and
-if that role was previously expanded (see L</expand> method), 
+if that role was previously expanded (see L</"expand"> method), 
 i.e. if the object hash contains an entry C<< $obj->{$role} >>, 
 then this data is reused instead of calling the database again.
 
@@ -546,9 +551,29 @@ To specify a unidirectional association, just supply
 to one of the role names. In that case the corresponding role
 method is not generated.
 
+When a role has multiplicity '*', another method
+named C<insert_into_...> is also installed, that will
+create new objects of the associated class, taking care
+of the linking automatically :
+
+  $anEmployee->insert_into_activities({d_begin => $today, dpt_id  => $dpt});
+
+This is equivalent to
+
+  Activity->insert({d_begin => $today, dpt_id  => $dpt, 
+                    emp_id  => $anEmployee->{emp_id}});
+
+
 The C<Association> method only supports binary associations; however,
 you can create a C<View> from a series of associations, in order
-to simultaneously join many tables : see method L</ViewFromRoles>.
+to simultaneously join many tables : see method L</"ViewFromRoles">.
+
+UML many-to-many associations (i.e. with '*' at both ends) are not
+supported : these are conceptual associations that cannot be
+implemented into a database without an intermediate linking table. As
+a matter of fact, the C<Activity> table of our example acts as a
+linking table implementing the conceptual many-to-many association
+between employees and departments.
 
 =cut
 
@@ -558,60 +583,86 @@ sub Association {
 
   $schema->isSchema or croak "'Association' is a schema class method";
 
-  my ($table1, $role1, $arity1, @cols1) = @$args1;
-  my ($table2, $role2, $arity2, @cols2) = @$args2;
+  my ($table1, $role1, $multipl1, @cols1) = @$args1;
+  my ($table2, $role2, $multipl2, @cols2) = @$args2;
 
   @cols1 == @cols2 or croak "Association: numbers of columns do not match";
   
+  not ($multipl1 =~ /\*/ and $multipl2 =~ /\*/) or
+    croak "Association : many to many is not supported";
+
   # for each side of the association ...
-  for ([$table1, $role1, $arity1, \@cols1, $table2, $arity2, \@cols2],
-       [$table2, $role2, $arity2, \@cols2, $table1, $arity1, \@cols1])
+  for ([$table1, $role1, $multipl1, \@cols1, $table2, $multipl2, \@cols2],
+       [$table2, $role2, $multipl2, \@cols2, $table1, $multipl1, \@cols1])
   {
-    my ($t, $r, $a, $c, $foreign_t, $foreign_a, $foreign_c) = @$_;
+    my ($table, $role, $multipl, $cols_ref, 
+	$foreign_table, $foreign_multipl, $foreign_cols_ref) = @$_;
 
-    next if not $r or $r =~ /^(0|""|''|none)$/;
+    next if not $role or $role =~ /^(0|""|''|none)$/; 
 
-    $t->classData->{classKind} eq 'Table' or 
-      croak "Association : $t is not a Table class";
+    $table->classData->{classKind} eq 'Table' or 
+      croak "Association : $table is not a Table class";
 
-    # build method as a closure 
-    my $meth = sub {
+    # build select method as a closure, and install it into foreign table
+    my $select_meth = sub {
       my $self = shift; 
-      ref($self) or croak "role $r cannot be called as class method";
+      ref($self) or croak "role $role cannot be called as class method";
 
       # if called without args, and that role was previously expanded,
       # then return the cached version
-      if ($self->{$r} and not @_) {
-	return $self->{$r};
+      if ($self->{$role} and not @_) {
+	return $self->{$role};
       }
 
       my %joinCols = ();
-      @joinCols{@$c} = @{$self}{@$foreign_c};
-      $t->preselectWhere(\%joinCols, $a)->(@_);
+      @joinCols{@$cols_ref} = @{$self}{@$foreign_cols_ref};
+      $table->preselectWhere(\%joinCols, $multipl)->(@_);
     };
+    _defineMethod($foreign_table, $role, $select_meth);
+
+
+    if ($multipl =~ /\*/) { # one to many, so install an "insert_into_" method
+      my $m_name = "insert_into_$role";
+
+      # build insert method as a closure, and install it into foreign table
+      my $insert_meth = sub {
+
+	my $self = shift; # remaining @_ contains refs to records for insert()
+	ref($self) or croak "$m_name cannot be called as class method";
+
+	# add join information into records that will be inserted
+	foreach my $record (@_) {
+	  not (grep {$_} @{$record}{@$cols_ref}) or
+	    croak "args to $m_name should not contain values in @$cols_ref";
+	  @{$record}{@$cols_ref} = @{$self}{@$foreign_cols_ref};
+	}
+
+	$table->insert(@_);
+      };
+      _defineMethod($foreign_table, $m_name, $insert_meth);
+    }
 
-    # install method
-    _defineMethod($foreign_t, $r, $meth);
 
     # record join parameters in schema->classData
     my %where;
-    @where{@$foreign_c} = @$c;
-    $schema->classData->{joins}{$foreign_t}{$r} = {
-      arity => $a,
-      table => $t,
-      where => \%where
+    @where{@$foreign_cols_ref} = @$cols_ref;
+    $schema->classData->{joins}{$foreign_table}{$role} = {
+      multiplicity => $multipl,
+      table        => $table,
+      where        => \%where,
      };
+
   }
 }
 
 
 =head4 ViewFromRoles
 
-  my $view = MySchema->ViewFromRoles($table => $role1 => $role2 => ..);
+  my $view = MySchema->ViewFromRoles($table, $role1, $role2, ..);
 
 Creates a C<View> class, starting from a given Table Class and
 following one or several associations through their role names.
-It calls the L<View> method, with a collection of parameters
+It calls the L</"View"> method, with a collection of parameters
 automatically inferred from the associations. So for example
 
   MySchema->ViewFromRoles(qw/Department activities employee/)
@@ -624,17 +675,34 @@ is equivalent to
     LEFT OUTER JOIN Employee ON Activity.emp_id   = Employee.emp_id
   _EOSQL_
   
-  MySchema->View(DepartmentActivitiesEmployee => '*' => $sql => 
+  MySchema->View(DepartmentActivitiesEmployee => '*', $sql, 
                  qw/Department Activity Employee/);
 
 For each pair of tables, the kind of join is chosen according to
-the arity declared with the role : if the minimum arity is 0, 
+the multiplicity declared with the role : if the minimum multiplicity is 0, 
 the join will be LEFT OUTER JOIN; otherwise it will be a usual inner join.
 (exception : after a first left join, all remaining tables are also
- connected through additional left joins).
+ connected through additional left joins). The default kind of join 
+chosen by this rule may be overriden by inserting pseudo-roles
+in the list, namely C<< '<=>' >> or C<< _INNER_ >> for inner joins 
+and C<< '=>' >> or C<< _LEFT_ >> for left joins. So for example 
+
+  MySchema->ViewFromRoles(qw/Department <=> >activities <=> employee/)
+
+becomes equivalent to 
+
+  my $sql = <<_EOSQL_
+  Department 
+    INNER JOIN Activity ON Department.dpt_id = Activity.dpt_id
+    INNER JOIN Employee ON Activity.emp_id   = Employee.emp_id
+  _EOSQL_
+
 
 The view name will be composed by concatenating the table and 
-the capitalized role names. Since such names might be long and uncomfortable 
+the capitalized role names. If join kinds were explicitly
+set, these also belong to the view name, like
+C<< Department_INNER_Activities >>.
+Since such names might be long and uncomfortable 
 to use, the view name is also returned as result of the method
 call, so that the client code can store it in a variable and use
 it as an alias.
@@ -679,6 +747,11 @@ sub ViewFromRoles {
 
   $self->isSchema or croak "ViewFromRoles must be called on a Schema class";
 
+  foreach (@roles) {
+    s[^(INNER|<=>)$] [_INNER_];
+    s[^(LEFT|=>)$]   [_LEFT_];
+  }
+
   my $viewName = join "", $table, map(ucfirst, @roles);  
   return $viewName if defined (%{$viewName.'::'}); # view was already generated
 
@@ -690,12 +763,27 @@ sub ViewFromRoles {
   my $joinInto = \@innerJoins; # initially; might change later to \@leftJoins
   my $curTable = $table;
   my ($dbTableLeft, $dbTableRight) = (undef, $table->table);
+  my $forcedJoin;
 
+ ROLE:
   foreach my $role (@roles) {
+
+    for ($role) {
+      /^_INNER_$/ and do {$forcedJoin = \@innerJoins; next ROLE;};
+      /^_LEFT_$/   and do {$forcedJoin = \@leftJoins;  next ROLE;};
+    }
+
     my $joinData = $self->schema->classData->{joins}{$curTable}{$role} or
       croak "ViewFromRoles: role $role not found in $curTable";
 
-    $joinInto = \@leftJoins if $joinData->{arity} =~ m/^(0|\*)/;
+    if ($forcedJoin) { 
+      $joinInto = $forcedJoin;
+      # THINK : maybe should not allow forced _INNER_ after an initial _LEFT_
+      $forcedJoin = undef;
+    }
+    elsif ($joinData->{multiplicity} =~ m/^(0|\*)/) {
+      $joinInto = \@leftJoins;
+    }
 
     $curTable = $joinData->{table};
     ($dbTableLeft, $dbTableRight) = ($dbTableRight, $curTable->table);
@@ -732,7 +820,7 @@ sub ViewFromRoles {
 
   # 3) install the View
 
-  $self->View($viewName => '*' => $sql => $where => @parentTables);
+  $self->View($viewName, '*', $sql, $where, @parentTables);
 }
 
 
@@ -746,12 +834,12 @@ sub ViewFromRoles {
 
 When applied to a schema class, this method declares a
 column type of name C<typeName>, to which a number of I<handlers> are
-associated (see methods L</ColumnHandlers> and L</applyColumnHandlers>).
+associated (see methods L</"ColumnHandlers"> and L</"applyColumnHandlers">).
 
 When applied to a table class, the handlers associated to 
 C<< typeName >> are registered for each of the columns
 C<< column1 >>, C<< column2 >> (throug a call to 
-L<ColumnHandlers>).
+L</"ColumnHandlers">).
 
 Such column types can be used for example for automatic conversion of values
 between database and memory, through 'fromDB' and 'toDB' handlers.
@@ -929,7 +1017,7 @@ Handler names 'fromDB' and 'toDB' have a special
 meaning : they are called automatically just after reading data from
 the database, or just before writing into the database.
 Handler name 'validate' is used by the method
-L<hasInvalidColumns>.
+L</"hasInvalidColumns">.
 
 
 =cut
@@ -1038,15 +1126,15 @@ Returns or sets the handle to a DBI database handle (see L<DBI>).
 This handle is schema-specific.
 C<DBIx::DataModel> expects the handle to be opened with
 C<< RaiseError => 1 >> 
-(see below L<Transactions and error handling>).
+(see below L</"Transactions and error handling">).
 
 =cut
 
 sub dbh {
   my ($self, $dbh) = @_;
   if ($dbh) {
-    $self->isSchema or croak "please call " . $self->schema . 
-                             "->dbh(..) to set the dbh handle";
+    $self->isSchema or croak "dbh(..) must be called on schema class";
+    $dbh->{RaiseError} or croak "arg to dbh(..) must have RaiseError=1";
     $self->classData->{dbh} = $dbh;
   }
   return $self->schema->classData->{dbh};
@@ -1106,20 +1194,20 @@ sub autoUpdateColumns {
 
   MySchema->selectImplicitlyFor('read only')
   MyClass ->selectImplicitlyFor('update')
-  $val = $myObj->selectImplicitlyFor;
+
 
 Gets or sets a default value for the C<-for> argument to 
-L</select()>. W
+L</"select">. W
 
 =cut
 
 sub selectImplicitlyFor {
-  my ($self) = @_;
+  my $self = shift;
 
-  if (exists($_[1])) {
+  if (@_) {
     not ref($self) 
       or croak "selectImplicitlyFor(value) : must be called as class method";
-    return $self->classData->{selectImplicitlyFor} = $_[1];
+    return $self->classData->{selectImplicitlyFor} = shift;
   }
   else {
     return exists($self->classData->{selectImplicitlyFor}) ? 
@@ -1134,6 +1222,35 @@ sub selectImplicitlyFor {
 
 =head3 Schema class methods
 
+
+=head4 tables
+
+Returns an array of names of 'Table' classes declared in this schema.
+
+=cut
+
+sub tables {
+  my ($self) = @_;
+  $self->isSchema or croak "tables() must be called on a Schema class";
+
+  return grep {$classData{$_}->{classKind} eq 'Table'} keys %classData;
+}
+
+
+=head4 views
+
+Returns an array of names of 'View' classes declared in this schema.
+
+=cut
+
+sub views {
+  my ($self) = @_;
+  $self->isSchema or croak "views() must be called on a Schema class";
+
+  return grep {$classData{$_}->{classKind} eq 'View'} keys %classData;
+}
+
+
 =head4 doTransaction
 
   MySchema->doTransaction(sub {...});
@@ -1254,7 +1371,7 @@ is usually '*'.
 
 the second argument C<< \%where >> is a reference to a hash or array of 
 criteria that will be translated into SQL clauses. In most cases, this
-will just be something like C<< {col1 => 'val1, col2 => 'val2} >>;
+will just be something like C<< {col1 => 'val1', col2 => 'val2'} >>;
 see L<SQL::Abstract::select> for a detailed description of the
 structure of that hash or array. It can also be
 a plain SQL string like C<< "col1 IN (3, 5, 7, 11) OR col2 IS NOT NULL" >>.
@@ -1267,9 +1384,9 @@ like C<< "col1 DESC, col3, col2 DESC" >>.
 
 =back
 
-If using named arguments, the C<<-for>> argument specifies an 
+If using named arguments, the C<< -for >> argument specifies an 
 additional clause to be added at the end of the SQL statement,
-like for example  C<<-for => 'read only'>>.
+like for example  C<< -for => 'read only' >>.
 
 No verification is done on the list of retrieved C<< \@columns >>,
 so it is OK if the list does not contain primary or foreign keys --- but then
@@ -1282,7 +1399,7 @@ passing join criteria. In that case, entries of the "moreWhere" hash are
 combined with the entries of the regular "where" hash before building
 the SQL statement. You only need to understand about this if you intend
 to write additional role methods; an example is shown in section
-L<Self-referential associations> below.
+L</"Self-referential associations"> below.
 
 =cut
 
@@ -1339,11 +1456,11 @@ sub selectSth {
 				   $args->{-orderBy});
   exists($args->{-for}) or $args->{-for} = $self->selectImplicitlyFor;
   $sql .= " FOR $args->{-for}" if $args->{-for};
-  $class->_debug($sql);
+  $class->_debug($sql . " / " . join(", ", @bind) );
   my $sth = $self->dbh->prepare($sql);
   $schemaData->{lasth} = $sth;
-  ($sth->execute(@bind) and return $sth)
-    or return undef; # in case of error
+  $sth->execute(@bind); # no check for return val because of RaiseError
+  return $sth;
 }
 
 =head4 preselectWhere
@@ -1361,17 +1478,17 @@ intend to write your own role methods.
 =cut
 
 sub preselectWhere {
-  my ($class, $where, $arity) = @_;
+  my ($class, $where, $multiplicity) = @_;
   return sub {
     my $selectArgs = &_parseSelectArgs;
     _addSelectCriteria($selectArgs, $where);
     my $result = $class->select(%$selectArgs);
-    if ($arity and $arity =~ /^([01]\.\.)?1$/) { # if maximum arity 1 
+    if ($multiplicity and $multiplicity =~ /^([01]\.\.)?1$/) { # if maximum multiplicity 1 
       @$result <= 1 or 
-	carp "too many results for arity $arity in class $class";
+	carp "too many results for multiplicity $multiplicity in class $class";
       return $result->[0];	   # return a single object
     }
-    else {                         # if maximum arity n
+    else {                         # if maximum multiplicity n
       return $result;              # return an array ref 
     }
   }
@@ -1407,7 +1524,7 @@ Inserts new records into the database, after having applied
 the 'toDB' handlers.  This operation 
 I<may modify the argument data> (manipulating values through 'toDB' handlers, 
 or deleting columns declared as 'noUpdate').
-See also section L<Transactions and error handling> below.
+See also section L</"Transactions and error handling"> below.
 
 =cut
 
@@ -1431,7 +1548,7 @@ sub insert {
     # now unbless $record into just a hashref and perform the insert
     bless $record, 'HASH';
     my ($sql, @bind) = $sqlA->insert($table, $record);
-    $class->_debug($sql);
+    $class->_debug($sql . " / " . join(", ", @bind) );
     my $sth = $self->dbh->prepare($sql);
     $self->schema->classData->{lasth} = $sth;
     $sth->execute(@bind);
@@ -1446,7 +1563,7 @@ sub insert {
   $record->applyColumnHandlers($handlerName);
 
 This is both a class and an instance method.  It looks for all columns
-having a handler named C<< $handlerName >> (see method L<ColumnHandlers>
+having a handler named C<< $handlerName >> (see method L</"ColumnHandlers">
 for how to declare handlers, and for the special handler names
 'fromDB', 'toDB' and 'validate'). Found handlers are then
 applied, either to the current object, or to the list of objects
@@ -1621,8 +1738,9 @@ sub hasInvalidColumns {
 
   $obj->expand($role [, @args] )
 
-Executes the method C<< $role >> to follow an Association, and 
-stores the result in the object itself under C<< $obj->{$role} >>.
+Executes the method C<< $role >> to follow an Association,
+stores the result in the object itself under C<< $obj->{$role} >>,
+and returns that result.
 This is typically used to expand an object into a tree datastructure.
 Optional C<< @args >> are passed to C<< $obj->$role(@args) >>.
 
@@ -1799,7 +1917,7 @@ sub _modifyData { # called by methods 'update' and 'delete'
   my ($sql, @bind) = ($toDo eq 'update') ? 
                         $schemaClassData->{sqlAbstr}->update($table, $self, \%where) :
 			$schemaClassData->{sqlAbstr}->delete($table, \%where);
-  $class->_debug($sql);
+  $class->_debug($sql . " / " . join(", ", @bind) );
   my $sth = $dbh->prepare($sql);
   $schemaClassData->{lasth} = $sth;
   $sth->execute(@bind);
@@ -2005,26 +2123,25 @@ associations :
                         [qw/Person none    * father_id/]);
 
 
-And there is a more sophisticated way to define the "children" method,
+And here is a more sophisticated way to define the "children" method,
 that will accept additional "where" criteria, like every regular method.
 
   package Person;
   sub children {
-    my $self = shift;
+    my $self = shift; # remaining args in @_ will be passed to select()
     my $id = $self->{pers_id};
     Person->preselectWhere([mother_id => $id, father_id => $id])->(@_);
   }
 
-This definition takes advantage of the hidden "moreWhere" argument
-of the L<select> method to force the join on C<mother_id> or 
+This definition forces the join on C<mother_id> or 
 C<father_id>, while leaving open the possibility for the caller
 to specify additional criteria. For example, all female children 
 of a person (either father or mother) can now be retrieved through
 
   $person->children({gender => 'F'})
 
-Observe the use of the C<-nest> operator from L<SQL::Abstract>
-to insert an arrayref (that will generate an SQL 'OR'). 
+Observe that C<mother_id> and C<father_id> are inside an arrayref
+instead of a hashref, so that L<SQL::Abstract> will generate an SQL 'OR'.
 
 =head1 SEE ALSO
 
@@ -2045,7 +2162,7 @@ L<http://www.officevision.com/pub/p5ee/>.
 
 =head1 TO DO 
 
-  - autodeclare tables and associations from $dbh->table_info, etc.
+  - autoloader to get tables and associations from $dbh->table_info, etc.
   - 'hasInvalidColumns' : should be called automatically before insert/update ?
   - 'validate' record handler
   - 'normalize' handler : for ex. transform empty string into null
